'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var babelPluginMacros = require('babel-plugin-macros');
var t = require('@babel/types');
var crypto = require('crypto');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var t__namespace = /*#__PURE__*/_interopNamespaceDefault(t);

// @ts-check
const scssCache = {};
const classCommentMather = /class:\s*([\w\d_-]+)/;
var css_macro = babelPluginMacros.createMacro(function ({ references, state, babel }) {
    if (!state.filename) {
        return;
    }
    const filename = state.filename;
    scssCache[filename] = '';
    for (const path of references.default) {
        if (!path.parentPath || !path.parentPath.isTaggedTemplateExpression()) {
            path.parentPath?.remove();
            continue;
        }
        const varPath = path.parentPath.parentPath;
        if (!varPath || !varPath.isVariableDeclarator()) {
            const quasi = path.parentPath.node.quasi;
            if (quasi.quasis[0].value.raw.trim()) {
                scssCache[filename] += quasi.quasis[0].value.raw + '\n';
            }
            if (path.parentPath.parentPath.isExpressionStatement()) {
                path.parentPath.parentPath.remove();
            }
            else {
                path.parentPath.replaceWith(t__namespace.stringLiteral(''));
            }
            continue;
        }
        let varName = '';
        if (t__namespace.isIdentifier(varPath.node.id)) {
            varName = varPath.node.id.name;
        }
        const leadingComments = varPath.parentPath.node.leadingComments;
        let isForceClass = false;
        if (leadingComments) {
            for (const comment of leadingComments) {
                const result = classCommentMather.exec(comment.value);
                if (result && result.length >= 2) {
                    varName = result[1];
                    isForceClass = true;
                    break;
                }
            }
        }
        if (path.scope.path.isFunctionDeclaration()) {
            varName = (path.scope.path.node.id?.name || '') + '-' + varName;
        }
        const quasi = path.parentPath.node.quasi;
        const id = isForceClass
            ? varName
            : 'styled-' + generateCssID(filename, varName);
        path.parentPath.replaceWith(t__namespace.stringLiteral(id));
        scssCache[filename] += `.${id} {${quasi.quasis[0].value.raw}}\n`;
    }
});
function generateCssID(filename, varName) {
    filename = filename.replace(process.cwd(), '');
    const h = crypto.createHash('sha256')
        .update(filename + varName)
        .digest('hex');
    return h.slice(0, 8);
}
function getScss(filename) {
    return scssCache[filename];
}
function getAllCacheScss() {
    return scssCache;
}

exports.default = css_macro;
exports.getAllCacheScss = getAllCacheScss;
exports.getScss = getScss;
