'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var babelPluginMacros = require('babel-plugin-macros');
var t = require('@babel/types');
var xmljs = require('xml-js');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var t__namespace = /*#__PURE__*/_interopNamespaceDefault(t);

// @ts-check
const xmlCache = {};
var xml_macro = babelPluginMacros.createMacro(function ({ references, state, babel }) {
    if (!state.filename) {
        return;
    }
    const filename = state.filename;
    for (const path of references.default) {
        if (!path.parentPath || !path.parentPath.isCallExpression()) {
            path.parentPath?.remove();
            continue;
        }
        const el = path.parentPath.node.arguments[0];
        if (!el || !t__namespace.isJSXElement(el)) {
            path.parentPath?.remove();
            continue;
        }
        const result = convertJSXtoXML(el);
        xmlCache[filename] = result;
        path.parentPath?.remove();
    }
});
function getXML(filename) {
    return xmlCache[filename];
}
function getAllCacheXML() {
    return xmlCache;
}
function formatXML(root) {
    return xmljs.js2xml({ elements: [root] }, { compact: false, spaces: '  ' });
}
/**
 * convert jsx to xml
 */
function convertJSXtoXML(root) {
    const el = { type: 'element' };
    const node = root.openingElement.name;
    let rootName = '';
    if (t__namespace.isJSXIdentifier(node)) {
        rootName = node.name;
    }
    const attrs = {};
    el.name = rootName;
    el.attributes = attrs;
    for (const attrNode of root.openingElement.attributes) {
        if (t__namespace.isJSXAttribute(attrNode) && attrNode.value) {
            if (t__namespace.isJSXIdentifier(attrNode.name) &&
                t__namespace.isStringLiteral(attrNode.value)) {
                attrs[attrNode.name.name] = attrNode.value.value;
            }
        }
    }
    if (root.children.length > 0) {
        const children = [];
        el.elements = children;
        for (const child of root.children) {
            if (t__namespace.isJSXElement(child)) {
                children.push(convertJSXtoXML(child));
            }
        }
    }
    return el;
}

exports.default = xml_macro;
exports.formatXML = formatXML;
exports.getAllCacheXML = getAllCacheXML;
exports.getXML = getXML;
